@@ -1,6 +1,6 @@
 /* 
    HTTP request/response handling
-   Copyright (C) 1999-2010, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2021, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -72,6 +72,8 @@ struct field {
     struct field *next;
 };
 
+/* Maximum number of interim responses. */
+#define MAX_INTERIM_RESPONSES (128)
 /* Maximum number of header fields per response: */
 #define MAX_HEADER_FIELDS (100)
 /* Size of hash table; 43 is the smallest prime for which the common
@@ -87,7 +89,7 @@ struct field {
 #define HH_HV_TRANSFER_ENCODING (0x07)
 
 struct ne_request_s {
-    char *method, *uri; /* method and Request-URI */
+    char *method, *target; /* method and request-target */
 
     ne_buffer *headers; /* request headers */
 
@@ -526,17 +528,18 @@ ne_request *ne_request_create(ne_session *sess,
     /* Only use an absoluteURI here when we might be using an HTTP
      * proxy, and SSL is in use: some servers can't parse them. */
     if (sess->any_proxy_http && !req->session->use_ssl && path[0] == '/')
-	req->uri = ne_concat(req->session->scheme, "://", 
-                             req->session->server.hostport, path, NULL);
+        req->target = ne_concat(req->session->scheme, "://",
+                                req->session->server.hostport,
+                                path, NULL);
     else
-	req->uri = ne_strdup(path);
+        req->target = ne_strdup(path);
 
     {
 	struct hook *hk;
 
 	for (hk = sess->create_req_hooks; hk != NULL; hk = hk->next) {
 	    ne_create_request_fn fn = (ne_create_request_fn)hk->fn;
-	    fn(req, hk->userdata, req->method, req->uri);
+	    fn(req, hk->userdata, req->method, req->target);
 	}
     }
 
@@ -724,7 +727,7 @@ void ne_request_destroy(ne_request *req)
     struct body_reader *rdr, *next_rdr;
     struct hook *hk, *next_hk;
 
-    ne_free(req->uri);
+    ne_free(req->target);
     ne_free(req->method);
 
     for (rdr = req->body_readers; rdr != NULL; rdr = next_rdr) {
@@ -759,7 +762,7 @@ void ne_request_destroy(ne_request *req)
 /* Reads a block of the response into BUFFER, which is of size
  * *BUFLEN.  Returns zero on success or non-zero on error.  On
  * success, *BUFLEN is updated to be the number of bytes read into
- * BUFFER (which will be 0 to indicate the end of the repsonse).  On
+ * BUFFER (which will be 0 to indicate the end of the response).  On
  * error, the connection is closed and the session error string is
  * set.  */
 static int read_response_block(ne_request *req, struct ne_response *resp, 
@@ -885,7 +888,8 @@ static ne_buffer *build_request(ne_request *req)
     ne_buffer *buf = ne_buffer_create();
 
     /* Add Request-Line and headers: */
-    ne_buffer_concat(buf, req->method, " ", req->uri, " HTTP/1.1" EOL, NULL);
+    ne_buffer_concat(buf, req->method, " ", req->target, " HTTP/1.1" EOL,
+                     NULL);
 
     /* Add custom headers: */
     ne_buffer_append(buf, req->headers->data, ne_buffer_size(req->headers));
@@ -913,7 +917,7 @@ static void dump_request(const char *request)
 	/* Display everything mode */
 	NE_DEBUG(NE_DBG_HTTP, "Sending request headers:\n%s", request);
     } else if (ne_debug_mask & NE_DBG_HTTP) {
-	/* Blank out the Authorization paramaters */
+	/* Blank out the Authorization parameters */
 	char *reqdebug = ne_strdup(request), *pnt = reqdebug;
 	while ((pnt = strstr(pnt, "Authorization: ")) != NULL) {
 	    for (pnt += 15; *pnt != '\r' && *pnt != '\0'; pnt++) {
@@ -1013,6 +1017,7 @@ static int send_request(ne_request *req, const ne_buffer *request)
     ne_status *const status = &req->status;
     int sentbody = 0; /* zero until body has been sent. */
     int ret, retry; /* retry non-zero whilst the request should be retried */
+    unsigned count;
     ssize_t sret;
 
     /* Send the Request-Line and headers */
@@ -1041,11 +1046,13 @@ static int send_request(ne_request *req, const ne_buffer *request)
     
     NE_DEBUG(NE_DBG_HTTP, "Request sent; retry is %d.\n", retry);
 
-    /* Loop eating interim 1xx responses (RFC2616 says these MAY be
-     * sent by the server, even if 100-continue is not used). */
-    while ((ret = read_status_line(req, status, retry)) == NE_OK 
-	   && status->klass == 1) {
-	NE_DEBUG(NE_DBG_HTTP, "Interim %d response.\n", status->code);
+    /* Loop eating interim 1xx responses; RFC 7231§6.2 says clients
+     * MUST be able to parse unsolicited interim responses. */
+    for (count = 0; count < MAX_INTERIM_RESPONSES
+             && (ret = read_status_line(req, status, retry)) == NE_OK
+             && status->klass == 1; count++) {
+	NE_DEBUG(NE_DBG_HTTP, "[req] Interim %d response %d.\n",
+                 status->code, count);
 	retry = 0; /* successful read() => never retry now. */
 	/* Discard headers with the interim response. */
 	if ((ret = discard_headers(req)) != NE_OK) break;
@@ -1058,6 +1065,10 @@ static int send_request(ne_request *req, const ne_buffer *request)
 	}
     }
 
+    if (count == MAX_INTERIM_RESPONSES) {
+        return aborted(req, _("Too many interim responses"), 0);
+    }
+
     return ret;
 }
 
@@ -1112,8 +1123,8 @@ static int read_message_header(ne_request *req, char *buf, size_t buflen)
 	
 	/* assert(buf[0] == ch), which implies len(buf) > 0.
 	 * Otherwise the TCP stack is lying, but we'll be paranoid.
-	 * This might be a \t, so replace it with a space to be
-	 * friendly to applications (2616 says we MAY do this). */
+	 * This might be a \t, so replace it with a space for ease of
+	 * parsing; this is permitted by RFC 7230§3.5. */
 	if (n) buf[0] = ' ';
 
 	/* ready for the next header. */
@@ -1323,23 +1334,22 @@ int ne_begin_request(ne_request *req)
     }
 
     /* Decide which method determines the response message-length per
-     * 2616§4.4 (multipart/byteranges is not supported): */
+     * RFC 7230§3.3.3, method cases follow: */
 
 #ifdef NE_HAVE_SSL
-    /* Special case for CONNECT handling: the response has no body,
-     * and the connection can persist. */
+    /* Case (2) is special-cased first for CONNECT: the response has
+     * no body, and the connection can persist. */
     if (req->session->in_connect && st->klass == 2) {
 	req->resp.mode = R_NO_BODY;
 	req->can_persist = 1;
     } else
 #endif
-    /* HEAD requests and 204, 304 responses have no response body,
-     * regardless of what headers are present. */
+    /* Case (1), HEAD requests and 204, 304 responses have no response
+     * body, regardless of what headers are present. */
     if (req->method_is_head || st->code == 204 || st->code == 304) {
     	req->resp.mode = R_NO_BODY;
     }
-    /* Broken intermediaries exist which use "transfer-encoding: identity"
-     * to mean "no transfer-coding".  So that case must be ignored. */
+    /* Case (3), chunked transer-encoding.. */
     else if ((value = get_response_header_hv(req, HH_HV_TRANSFER_ENCODING,
                                              "transfer-encoding")) != NULL
              && ne_strcasecmp(value, "identity") != 0) {
@@ -1351,7 +1361,8 @@ int ne_begin_request(ne_request *req)
         else {
             return aborted(req, _("Unknown transfer-coding in response"), 0);
         }
-    } 
+    }
+    /* Case (4) and (5), content-length delimited. */
     else if ((value = get_response_header_hv(req, HH_HV_CONTENT_LENGTH,
                                              "content-length")) != NULL) {
         char *endptr = NULL;
@@ -1360,11 +1371,14 @@ int ne_begin_request(ne_request *req)
         if (*value && len != NE_OFFT_MAX && len >= 0 && endptr && *endptr == '\0') {
             req->resp.mode = R_CLENGTH;
             req->resp.body.clen.total = req->resp.body.clen.remain = len;
-        } else {
-            /* fail for an invalid content-length header. */
+        }
+        else {
+            /* Per case (4), an invalid C-L must be treated as an error. */
             return aborted(req, _("Invalid Content-Length in response"), 0);
         }
-    } else {
+    }
+    /* Case (7), response delimited by EOF. */
+    else {
         req->resp.mode = R_TILLEOF; /* otherwise: read-till-eof mode */
     }
     
