@@ -58,7 +58,7 @@ static const struct {
     { NULL }
 };
 
-static const char *bad_sl[] = {
+static const char *const bad_sl[] = {
     "",
     "HTTP/1.1 1000 OK",
     "HTTP/1.1 1000",
@@ -165,14 +165,40 @@ static int md5_alignment(void)
     return OK;
 }
 
+#define INIT_MD5 "0123456789abcdeffedcba9876543210"
+
+static int md5_read(void)
+{
+    union {
+        unsigned int int32[4];
+        unsigned char buf[16];
+    } u;
+    struct ne_md5_ctx *ctx = ne_md5_create_ctx();
+    void *rv;
+    char hex[33];
+
+    rv = ne_md5_read_ctx(ctx, u.buf);
+    ONN("bogus return value", rv != u.buf);
+
+    ne_md5_to_ascii(u.buf, hex);
+
+    ONV(strcmp(INIT_MD5, hex) != 0,
+        ("read context was %s not %s", hex, INIT_MD5));
+
+    ne_md5_destroy_ctx(ctx);
+
+    return OK;
+}
+
 static const struct {
     const char *str;
     time_t time;
-    enum { d_rfc1123, d_iso8601, d_rfc1036 } type;
+    enum { d_rfc1123, d_iso8601, d_rfc1036, d_asctime } type;
 } good_dates[] = {
     { "Fri, 08 Jun 2001 22:59:46 GMT", 992041186, d_rfc1123 },
     { "Friday, 08-Jun-01 22:59:46 GMT", 992041186, d_rfc1036 },
     { "Wednesday, 06-Jun-01 22:59:46 GMT", 991868386, d_rfc1036 },
+    { "Wed Jun 06 22:59:46 2001", 991868386, d_asctime },
     /* some different types of ISO8601 dates. */
     { "2001-06-08T22:59:46Z", 992041186, d_iso8601 },
     { "2001-06-08T22:59:46.9Z", 992041186, d_iso8601 },
@@ -193,6 +219,7 @@ static int parse_dates(void)
 	case d_rfc1036: res = ne_rfc1036_parse(str); break;
 	case d_iso8601: res = ne_iso8601_parse(str); break;
 	case d_rfc1123: res = ne_rfc1123_parse(str); break;
+	case d_asctime: res = ne_asctime_parse(str); break;
 	default: res = -1; break;
 	}
 	
@@ -316,6 +343,7 @@ ne_test tests[] = {
     T(status_lines),
     T(md5),
     T(md5_alignment),
+    T(md5_read),
     T(parse_dates),
     T(bad_dates),
     T(versioning),
